// C Source File
// Created 6/7/00; 3:07:29 PM

/*---------------------------Painter 89----------------------------------
This is a paint program for the Ti-89, written in C using TIGCC and Zeljko
Juric's TIGCC 2.1 library. The files "keycodes.h" and "painter.h" should have
come with this file. This source is well commented, but there are some advanced
things in here (like double buffering), so don't be discouraged if you don't 
understand it all. Double buffering is a technique that allows you to draw
to a virtual screen, and quickly copy it back to the main screen, so you can't
see the drawing. It's also useful if you want to have a background that isn't
corrupted by animation on the screen (which is why it is used here). This 
version of Painter now has support for saving/loading pictures (with 
compression, using ziplib).

If you have any questions, e-mail me at freakonaleash881@hotmail.com

Michael Cowart
-------------------------------------------------------------------------*/

   #define USE_GRAY_PATCH
// #define USE_LONGMUL_PATCH
// #define USE_LONGDIV_PATCH
// #define USE_INITS_PATCH
	 
#include <doors.h>   	// Make Doors Program
#include <all.h>     	// Include All Header Files
#include "keycodes.h"	// Ti-89/92+ Keycodes
#include "painter.h" //the guts of the program

int _ti89;           // Produce .89Z File
int _ti92plus;			 // Produce .9xz File

char _comment[] = "Painter version 2.0";

//must have ziplib
int _ziplib;

//different screen size values for ti89, 92+
int max_x;
int max_y;


// Main Function
int _main()
{
	//variable used to store keypress in
	int key=0;
	
	//starting coordinates
	int x=0,y=0;

	//start parameters
	int color=BLACK;
	int tcolor;
	int size=3;
	int horz=3;
	int vert=3;
	int mode=NONE;
	int x1=0,y1=0;
	
	//determines if calc is 89 or 92+, and sets screen values accordingly
	if (TI89 == TRUE)
	{
		max_x = 160;
		max_y = 100;
	}
	else
	{
		max_x = 240;
		max_y = 128;
	}
			
	//game initialization
	GameInit ();		
	DrawCursor (x,y,size);
			
	/*main loop (when you hit Escape, the program terminates). All statements in
	here have to do with keypresses. Note the use of pushkey (0). This clears the
	keyboard cache, which is IMPORTANT!*/
	
	while (key!=KEY_ESCAPE)
	{
		key = GetKeyWithAPD ();
													
			//turns drawing on/off
			if (key==KEY_X)
			{
				pushkey (0);
				//determines on/off status
				switch (mode)
				{
					//if off, turns it on
					case NONE:
					{
						DrawPen (x,y,size,color);
						PageFlip ();
						DrawCursor (x,y,size);
						mode=DRAW;
						break;
					}
					
					//if on, turns it off
					case DRAW:
					{
					mode=NONE;
					break;
					}
				}
			}	
			
			//erases the area under the cursor
			if (key==KEY_Y)
			{
				pushkey (0);
				if (mode==NONE)
				{
					//saves current color
					tcolor=color;
					
					//erase area under cursor
					color=ERASE;
					DrawPen (x,y,size,color);
					PageFlip ();
					DrawCursor (x,y,size);
					
					//restores color
					color=tcolor;
				}
			}
			
			//turns circle drawing on/off
			if (key==KEY_APPS)
			{
				pushkey (0);
				//determines on/off status
				switch (mode)
				{
					//if off, turns it on
					case NONE:
					{
						PageFlip ();
						DrawEllipse (x,y,horz,vert);
						mode=CIRCLE;
						break;
					}
				
					//if on, turns it off
					case CIRCLE:
					{
						ColorEllipse (x,y,horz,vert,color);
						PageFlip();
						mode=NONE;
						DrawCursor (x,y,size);
						horz=2;
						vert=2;
						break;
					}
				}
			}
			
			//turns line drawing on/off
			if (key==KEY_ENTER)
			{
				pushkey (0);
				//determines on/off status
				switch (mode)
				{
					//if off, turn on
					case NONE:
					{
						x1=x+2;
						y1=y;
						PageFlip ();
						MyDrawLine(x,y,x1,y1);
						mode=LINE;
						break;
					}
				
					//if on, turn off
					case LINE:
					{
						ColorLine (x,y,x1,y1,color);
						PageFlip ();
						DrawCursor (x,y,size);
						mode=NONE;
						break;
					}
				}
			}
			
			//increases the cursor size in normal and draw modes
			if (key==KEY_PLUS)
			{
				pushkey (0);
				//if size is too big, cancels resizing
				if (size < 12)
				{
					switch (mode)
					{
						//don't do anything (in circle mode)
						case CIRCLE:
						{break;}
						
						//ditto (in line mode)
						case LINE:
						{break;}
						
						default:
						{
							size+=3;
							if (mode==DRAW)
							DrawPen (x,y,size,color);
							//copies virtual screen to real one			
							PageFlip ();					
							DrawCursor (x,y,size);
						}
					}
				}
			}
			
			//decreases cursor size in normal and draw modes
			if (key==KEY_MINUS)
			{
				pushkey (0);
				//if size is too small, cancels resizing
				if (size > 3)
				{
					switch (mode)
					{
						//don't do anything (in circle mode)
						case CIRCLE:
						{break;}
						
						//don't do anything (in line mode)
						case LINE:
						{break;}
						
						default:
						{
							size-=3;
							if (mode==DRAW)
							DrawPen (x,y,size,color);
							//copies virtual screen to real one			
							PageFlip ();					
							DrawCursor (x,y,size);
						}
					}
				}
			}
			
			//changes color (to light grey)
			if (key==KEY_F1)
			{
				pushkey (0);
				color=LGREY;
				PortSet (plane1,239,127);
				FontSetSys (F_4x6);
				DrawStr (100,max_y-6,"Light Grey",A_NORMAL);
				if (mode==DRAW)
				DrawPen (x,y,size,color);
			}
			
			//changes color (to dark grey)
			if (key==KEY_F2)
			{
				pushkey (0);
				color=DGREY;
				PortSet (plane1,239,127);
				FontSetSys (F_4x6);
				DrawStr (100,max_y-6,"Dark Grey",A_NORMAL);
				if (mode==DRAW)
				DrawPen (x,y,size,color);
			}
			
			//changes color (to black)
			if (key==KEY_F3)
			{
				pushkey (0);
				color=BLACK;
				PortSet (plane1,239,127);
				FontSetSys (F_4x6);
				DrawStr (100,max_y-6,"Black",A_NORMAL);
				if (mode==DRAW)
				DrawPen (x,y,size,color);
			}
			
			//save a file
			if (key==KEY_F4)
			{
				SavePic ();
				PageFlip ();
				DrawCursor (x,y,size);
			}
			
			//load a file
			if (key==KEY_F5)
			{
				LoadPic ();
				PageFlip ();
				DrawCursor (x,y,size);
			}
			
			//clears the screen
			if (key==KEY_CLEAR)
			{
				GreyClrScr ();
				mode=NONE;
				DrawCursor (x,y,size);
			}
			
			//deals with cursor movement
			if (key==KEY_UP)
			{
				pushkey (0);
				if (y > 0)
				{
					switch (mode)
					{
						case LINE:	
						{
							y1-=2;
							break;
						}
						
						case CIRCLE:
						{
							if (vert > 3)
							vert-=3;
							break;
						}
						
						default:
						{y-=2;}
					}
					DrawChoose (x,y,x1,y1,size,color,horz,vert,mode);
				}
			}		
			
			if (key==KEY_DOWN)
			{
				pushkey (0);
				if (y < max_y)
				{
					switch (mode)
					{
						case LINE:	
						{
							y1+=2;
							break;
						}
						
						case CIRCLE:
						{
							vert+=3;
							break;
						}
						
						default:
						{y+=2;}
					}
					DrawChoose (x,y,x1,y1,size,color,horz,vert,mode);
				}
			}
			
			if (key==KEY_RIGHT)
			{
				pushkey (0);
				if (x < max_x)
				{
					switch (mode)
					{
						case LINE:	
						{
							x1+=2;
							break;
						}
						
						case CIRCLE:
						{
							horz+=3;
							break;
						}
						
						default:
						{x+=2;}
					}
					DrawChoose (x,y,x1,y1,size,color,horz,vert,mode);
				}
			}
			
			if (key==KEY_LEFT)
			{
				pushkey (0);
				if (x > 0)
				{
					switch (mode)
					{
						case LINE:	
						{
							x1-=2;
							break;
						}
						
						case CIRCLE:
						{
							if (horz > 3)
							horz-=3;
							break;
						}
						
						default:
						{x-=2;}
					}
					DrawChoose (x,y,x1,y1,size,color,horz,vert,mode);
				}
			}
			
		}//end of the main loop
	
	//cleans everything up
	GameShutDown ();
		
	return 0;
}//end of the program
